using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.Diagnostics;

namespace Coursework
{
    //sound effects. used to select which clip to play
    public enum AudioClips : int
    {
        FOOTSTEPS = 0,
        FOOTSTEPS2,
        STAIRS,
        SELECT
    };

    //songs. used to select which song to play
    public enum AudioSongs : int
    {
        MAINGAME = 0,
        INTRO
    };

    //game states. used to control the screens of the game
    public enum GameStates : int
    {
        INTRO = 0,
        MAINGAME
    };

    /// <summary>
    /// Main game class
    /// Handles loading, rendering and sound
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        //graphics device
        GraphicsDeviceManager graphics;
 
        //sprite batch for 2d rendering of sprites and fonts
        SpriteBatch spriteBatch;
        SpriteFont fontToUse;

        //effects
        BasicEffect basicEffect;
        Effect testEffect; //custom lighting/bump mapping
        Effect postEffect; //post processing/invert shader
        String[] postTechniques; //stores the names all the techniques
        int currentPostTech; //current technique rendering

        //Camera
        Camera CamFree;
        Camera CamGame;
        Camera activeCamera; //current camera used
        Matrix projectionMatrix;
        Matrix worldMatrix;
        private float aspectRatio; //screen ratio

        //textures
        //general textures
        Texture2D intro;
        Texture2D whiteRect;
        Texture2D textBoxTexture;
        //maze textures
        Texture2D brickTexture;
        Texture2D brickTextureNormal;
        //stairTextures
        Texture2D stairTexture, stairTextureNormal;
        //character textures
        Texture2D charTexture, charTextureNormal;
        //level texture (currrent texture for the maze model)
        Texture2D levelTexture;
        Texture2D levelTextureNormal;

        //keyboard and gamepad states
        KeyboardState oldKeyBoardState, newKeyboardState;
        GamePadState oldGamePadState, newGamePadState;

        //models
        Model straightModel, junctionModel, deadEndModel, cornerModel, fourwayModel;
        Model stairs;
        Model character;

        //player/user
        PlayerModel gamePlayer;

        //fog
        Fog foggy;

        //map
        Map GameMap;

        //fps
        float fps;
        float deltaTime;
        int fpsCounter;

        //used to stop user input over a period of time
        float waitValue;
        float waitTime;

        //toggle controls
        bool runInFullScreen = false;
        bool drawWireframe = false;
        bool drawMap = false;
        bool drawBox = false;
        bool soundOn = true;
        bool useCustomShader = false;
        bool showDebugText = false;
        bool imageShader = true;

        //quick fix for xbox
        bool loadSound = true;

        //screen the game is on
        GameStates currentState = GameStates.INTRO;

        //light class
        Lighting basicLighting;

        //sound
        //full songs
        Song[] fullSongArray;
        //sound effects
        SoundEffectInstance[] audioClipInstance;
        SoundEffect[] audioClipArray;
        //controls footsteps. left right left right etc
        bool leftFoot;

        //text box to display text
        TextBox boxOfText;

        //render to a texture for post processing effects
        RenderTarget2D renderImage;       

        //uses spriteBatch to draw any debug text to the screen
        private void writeDebugText()
        {
            
                //begin
                //save state mode is used so the 3d models and 2d text/images can be drawn at the same time
                spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
                    
                    //output current cam type
                    if (activeCamera.getRestrictedState())
                    {
                        //set the position of the text and the string to display
                        Vector2 camOutPos = new Vector2(10, 10);
                        string camText = "Game Cam";
                        //use DrawString to render to the screen
                        spriteBatch.DrawString(fontToUse, camText, camOutPos, Color.Red);
                    }

                    else
                    {
                        //set the position of the text and the string to display
                        Vector2 camOutPos = new Vector2(10, 10);
                        string camText = "Free Cam";
                        //use DrawString to render to the screen
                        spriteBatch.DrawString(fontToUse, camText, camOutPos, Color.Red);
                    }

                    //output player direction and player coordinates                    
                    Vector2 dirOutputPos = new Vector2(10, 320); //text position
                    string dirOutputText = "Facing: " + GameMap.getPlayerDirection(); //text
                    spriteBatch.DrawString(fontToUse, dirOutputText, dirOutputPos, Color.White); //draw
                    dirOutputPos.Y += 20; //move the position for the next line
                    string posOutputText = "X: " + GameMap.getPlayerX() + " Y: " + GameMap.getPlayerY(); //text
                    spriteBatch.DrawString(fontToUse, posOutputText, dirOutputPos, Color.White); //draw

                    //output camera position and target
                    //string.Format used to limited the number of digits displayed
                    Vector2 camareOutputPos = new Vector2(10, 400);
                    string camOutputText = "Cam X - " + String.Format("{0:0.##}",activeCamera.getCamPosition().X) + "\nCam Y - " + String.Format("{0:0.##}",activeCamera.getCamPosition().Y) + "\nCam Z - " + String.Format("{0:0.##}",activeCamera.getCamPosition().Z);
                    spriteBatch.DrawString(fontToUse, camOutputText, camareOutputPos, Color.White);
                    Vector2 targetOutputPos = new Vector2(10, 480);
                    string targetOutputText = "Target X - " + String.Format("{0:0.##}",activeCamera.getTarget().X) + "\nTarget Y - " + String.Format("{0:0.##}",activeCamera.getTarget().Y) + "\nTarget Z - " + String.Format("{0:0.##}",activeCamera.getTarget().Z);
                    spriteBatch.DrawString(fontToUse, targetOutputText, targetOutputPos, Color.White);
                    
                    //output the fps (should always sit around 60)
                    Vector2 FPSOutputPos = new Vector2(400, 10);
                    string FPSOutputText = "FPS: " + fps;
                    spriteBatch.DrawString(fontToUse, FPSOutputText, FPSOutputPos, Color.Red);
                
                //end
                spriteBatch.End();
        }

        /// <summary>
        /// Ensures a key can only be pressed ones, preventing muliple triggers when the key is held down
        /// </summary>
        /// <param name="checkKey">Current key pressed</param>
        /// <returns>returns true if the key can be used</returns>
        private bool singleUseKey(Keys checkKey)
        {
            //check the new and old game states
            //a key can only be used if it has been pressed down, then released
            if (newKeyboardState.IsKeyUp(checkKey) && oldKeyBoardState.IsKeyDown(checkKey))
                return true;

            return false;
        }

        //ensures a button can only be pressed once, preventing it from triggering multiple times when held done
        private bool singleUseButton()
        {     

            //check the packet numbers of the new and old state
            //if the packets are not the same then a different button has been pressed
            if (newGamePadState.PacketNumber != oldGamePadState.PacketNumber)
                return true;

            return false;
        }

        /// <summary>
        /// Initializes the classes required and sets initial properties
        /// </summary>
        public Game1()
        {
            //gfx
            graphics = new GraphicsDeviceManager(this);
            graphics.IsFullScreen = runInFullScreen;
            graphics.PreferMultiSampling = false; //changed for rendertarget


            //gfx pipeline management thingy
            Content.RootDirectory = "Content";

            //fps variables
            fps = 0;
            deltaTime = 0;
            fpsCounter = 0;

            //mouse
            this.IsMouseVisible = false;

            //game levels
            GameMap = new Map();
            GameMap.initMap();

            //free cam
            CamFree = new Camera();
            CamFree.initCam(Vector3.Zero, Looking.RIGHT, GameMap.getPlayerDirection(), false);

            //game cam
            CamGame = new Camera();
            CamGame.initCam(Vector3.Zero, Looking.RIGHT, GameMap.getPlayerDirection(), true);

            //set the active camera
            activeCamera = CamGame;          

            //player
            gamePlayer = new PlayerModel();
            gamePlayer.initPlayerModel(activeCamera.getCamPosition(), GameMap.getPlayerDirection(), activeCamera.getRestrictedState());

            //fog
            foggy = new Fog();
            foggy.initFog(60.0f, 80.0f, Color.AliceBlue.ToVector3(), true);

            //lighting for basic lighting
            basicLighting = new Lighting();
            basicLighting.initLight0(true, new Vector3(0.8f, 0.8f, 0.8f), new Vector3(0.0f, 1.0f, 0.0f), new Vector3(1, 1, 1));
            basicLighting.initLight1(true, new Vector3(0.8f, 0.8f, 0.8f), new Vector3(0.0f, 1.0f, 0.0f), new Vector3(1, 1, 1));
            basicLighting.initLight2(true, new Vector3(0.7f, 0.7f, 0.7f), new Vector3(0.0f, 1.0f, 0.0f), new Vector3(1, 1, 1));       
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            //create the basic effect to render with
            basicEffect = new BasicEffect(graphics.GraphicsDevice, null);

            //init the render target
            //since all the post processing is the same size as the screen the back buffer values are passed in
            renderImage = new RenderTarget2D(graphics.GraphicsDevice,
                graphics.GraphicsDevice.PresentationParameters.BackBufferWidth,
                graphics.GraphicsDevice.PresentationParameters.BackBufferHeight,
                1, //mipmap
                graphics.GraphicsDevice.PresentationParameters.BackBufferFormat
                );
          
            //textbox used to display the results of any user interation
            boxOfText = new TextBox();
            int tempWidth = 500; //temp values used to the basic properties of the box
            int tempHeight = 150;
            boxOfText.initTextBox((graphics.GraphicsDevice.Viewport.Width/2) - (tempWidth/2), (graphics.GraphicsDevice.Viewport.Height-tempHeight-50), tempWidth, tempHeight, Color.DodgerBlue);
            boxOfText.initBorder(3, Color.Black);
            boxOfText.initTextProperties(Color.White, 5, 5);

            //string array with the names of thge post processing effect
            //allows the post processing technique to be set using an array value
            postTechniques = new String[] { "Invert", "Blur", "Sepia", "BlackAndWhite", "NightVision" };
            currentPostTech = 0; //current position in the PP array
            
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// 
        /// called by initialize
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            //aspect ratio is used to represent the screen width/height ratio
            aspectRatio = graphics.GraphicsDevice.Viewport.AspectRatio;
            //standard projection
            projectionMatrix = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f), aspectRatio, 1.0f, 10000.0f);
            //projectionMatrix = Matrix.CreateOrthographic(800, 600, 1.0f, 1000.0f);

            //fonts
            fontToUse = Content.Load<SpriteFont>(".\\Fonts\\army");

            //models and model textures           
            //maze
            cornerModel = Content.Load<Model>("Model\\corner");
            deadEndModel = Content.Load<Model>("Model\\deadEnd");
            junctionModel = Content.Load<Model>("Model\\junction");
            fourwayModel = Content.Load<Model>("Model\\fourway");
            straightModel = Content.Load<Model>("Model\\straight");
            brickTexture = Content.Load<Texture2D>("Textures\\brick");
            brickTextureNormal = Content.Load<Texture2D>("Textures\\brickNormal");

            //stairs
            stairs = Content.Load<Model>("Model\\smallStairs");
            stairTexture = Content.Load<Texture2D>("Textures\\smallStairsColour");
            stairTextureNormal = Content.Load<Texture2D>("Textures\\smallStairsNormal");

            //character
            character = Content.Load<Model>("Model\\ammonite");
            charTexture = Content.Load<Texture2D>("Textures\\shellColour");
            charTextureNormal = Content.Load<Texture2D>("Textures\\shellNormal");

            //other textures    
            //xna does not seem to have a draw rectangle function so we'll use a textures instead
            whiteRect = Content.Load<Texture2D>("Textures\\whiteRect");
            textBoxTexture = Content.Load<Texture2D>("Textures\\textBox");
            intro = Content.Load<Texture2D>("Textures\\intro");

            //shaders 
            testEffect = Content.Load<Effect>("Shaders\\ShaderTest"); //custom lighting
            postEffect = Content.Load<Effect>("Shaders\\ImageShader"); //post processing          

            //load some sound
            //full songs
            fullSongArray = new Song[5];
          
            //audio clips
            //must match the enum
            String[] clipNames = new String[] { "singleFootstep", "singleFootstep2", "stairs", "select"};
            audioClipArray = new SoundEffect[clipNames.Length];
            //create one instance for each sound
            audioClipInstance = new SoundEffectInstance[clipNames.Length];
            //loop through the sound effect names and use the string value to load them in
            for (int i = 0; i < clipNames.Length; i++)
            {
                audioClipArray[i] = Content.Load<SoundEffect>("Audio\\" + clipNames[i]);
                audioClipInstance[i] = audioClipArray[i].CreateInstance();
                audioClipInstance[i].IsLooped = false; //sound effects are one shot
            }

            //pan the foot steps
            //-1.0 left, 1.0 right
            audioClipInstance[0].Pan = -0.4f; //left footstep
            audioClipInstance[1].Pan = 0.4f; //right footstep
            audioClipInstance[3].Volume = 0.2f; //climbing stairs

            //full songs
            //must match the enum
            String[] songNames = new String[] { "DenOndeKysten", "HanSomReiste" };
            fullSongArray = new Song[songNames.Length];
            //loop through the song names and use the string value to load them in
            for (int i = 0; i < songNames.Length; i++)
            {
                //mp3 uses different thing
                fullSongArray[i] = Content.Load<Song>("Audio\\" + songNames[i]);   
            }       

            //Set up the level textures
            //this can be used to change the model textures when the player reaches a new level
            levelTexture = brickTexture;
            levelTextureNormal = brickTextureNormal;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            //probably as accurate as counting mississippi out loud
            fpsCounter++;
            deltaTime += gameTime.ElapsedRealTime.Milliseconds; 

            //update every second
            if (deltaTime >= 1000)
            {
                fps = fpsCounter; //the fps is the number the counter was incremented by
                deltaTime = 0;
                fpsCounter = 0;
            }            

            //the wait time will always increment
            //the game will only ever wait if the wait value has been set
            waitTime += gameTime.ElapsedRealTime.Milliseconds;
            //if we dont have to wait, we can check the user input
            if (waitTime >= waitValue || waitTime>= float.MaxValue) 
            {
                //the player might get load in the maze for a VERY long time 
                //so waitTime has to be tested against is maximun value            
                waitTime = 0;
                waitValue = 0;
                CheckAction();
            }
     
            //quick fix
            //the xbox did not like the song being set to play during the initialization and it would freeze the games controls - no idea why
            //although this adds extra work every update call, it fixes the problem
            if (soundOn)
            {
                if (loadSound)
                {
                    loadSound = false; //set to never trigger again
                    playSong(AudioSongs.MAINGAME);
                    setSongVolume(0.3f);
                }
            }       

            //update the camera
            activeCamera.update();


            //if the model is not to be drawn, then it is following the game camera so update it position
            if (!gamePlayer.getDrawModel())
            {
                gamePlayer.updateModel(activeCamera.getCamPosition(), GameMap.getPlayerDirection());
            }      
  
            base.Update(gameTime);

         
        }

        public void CheckAction()
        {
            //check key
            newKeyboardState = Keyboard.GetState();
            newGamePadState = GamePad.GetState(PlayerIndex.One);

            if (currentState == GameStates.INTRO)
            {
                //controls
                if (singleUseKey(Keys.Enter) || (newGamePadState.Buttons.Start == ButtonState.Pressed && singleUseButton()))
                {
                   currentState = GameStates.MAINGAME;
                }

            }
            

            else
            {
                //////////////////////////////////////////////
                // Anytime Keys
                //////////////////////////////////////////////

                // Allows the game to exit
                if (newKeyboardState.IsKeyDown(Keys.Escape) || newGamePadState.Buttons.Back == ButtonState.Pressed)
                    this.Exit();

                //fullscreen
                if (singleUseKey(Keys.F1) || singleUseKey(Keys.D1))
                {
                    graphics.ToggleFullScreen();

                }
                //wireframe 
                if (singleUseKey(Keys.F2) || singleUseKey(Keys.D2))
                {
                    if (!imageShader)
                    {
                        if (drawWireframe)
                            drawWireframe = false;
                        else
                            drawWireframe = true;
                    }
                    }

                //Turn sound on or off
                if (singleUseKey(Keys.F3) || singleUseKey(Keys.D3) || (newGamePadState.Buttons.LeftShoulder == ButtonState.Pressed && singleUseButton()))
                {
                    if (soundOn)
                    {
                        soundOn = false;
                        pauseSong();
                    }
                    else
                    {
                        soundOn = true;
                        playSong(AudioSongs.MAINGAME);
                        setSongVolume(0.3f);
                    }
                }

                //show debug text
                if (singleUseKey(Keys.F4) || singleUseKey(Keys.D4))
                {
                    if (showDebugText)
                    {
                        showDebugText = false;
                    }
                    else
                    {
                        showDebugText = true;
                    }
                }

                //post processing on/off 
                if (singleUseKey(Keys.F8) || singleUseKey(Keys.D8))
                {
                    if (imageShader)
                    {
                        imageShader = false;
                    }
                    else
                    {
                        imageShader = true;
                        //make sure wireframe is off
                        if (drawWireframe)
                        {
                            graphics.GraphicsDevice.RenderState.FillMode = FillMode.Solid;
                            drawWireframe = false;
                        }
                    }
                }

                //customShader 
                if (singleUseKey(Keys.F9) || singleUseKey(Keys.D9))
                {
                    if (useCustomShader)
                    {
                        //taken out cos it breaks the game
                        //useCustomShader = false;
                    }
                    else
                    {
                        useCustomShader = true;
                    }
                }

                //last minute controls added to flick through shaders. i wrote them so might as well put them in
                if (singleUseKey(Keys.F7) || singleUseKey(Keys.D7))
                {
                    //only flick through if its turned on
                    if (imageShader)
                    {
                        if (currentPostTech < postTechniques.Length -1)
                            currentPostTech++;
                        else
                            currentPostTech = 0;
                    }
                }



                //switch camera
                if (singleUseKey(Keys.Q) || (newGamePadState.Buttons.RightShoulder == ButtonState.Pressed && singleUseButton()))
                {
                    if (activeCamera == CamFree)
                    {
                        activeCamera = CamGame;
                    }

                    else
                    {
                        activeCamera = CamFree;
                    }

                    //active camera has been changed so update the model to show or now
                    gamePlayer.setDrawModel(activeCamera.getRestrictedState());

                }

                //draw the HUD
                if (singleUseKey(Keys.Tab) || (newGamePadState.Buttons.Y == ButtonState.Pressed && singleUseButton()))
                {
                    if (drawMap)
                    {
                        drawMap = false;
                    }
                    else
                    {
                        drawMap = true;
                    }
                }

                //controls
                if (singleUseKey(Keys.Enter) || (newGamePadState.Buttons.Start == ButtonState.Pressed && singleUseButton()))
                {
                        currentState = GameStates.INTRO;                    
                }


                //////////////////////////////////////////////
                // Game Cam Keys
                //////////////////////////////////////////////

                if (activeCamera.getRestrictedState())
                {
                    //move forwared
                    if (newKeyboardState.IsKeyDown(Keys.Up) || newGamePadState.DPad.Up == ButtonState.Pressed)
                    {
                        if (activeCamera.getStillToMove() <= 0)
                        {
                            if (GameMap.mapCollision())
                            {
                                activeCamera.setStillToMove(GameMap.getPlayerDirection());
                                GameMap.updatePlayerPosition();

                                if (drawBox)
                                    drawBox = false;

                                //bah, not happy with this method, but works
                                if (leftFoot)
                                {
                                    playAudioClip(AudioClips.FOOTSTEPS);
                                    leftFoot = false;
                                }
                                else
                                {
                                    playAudioClip(AudioClips.FOOTSTEPS2);
                                    leftFoot = true;
                                }
                            }
                        }



                    }

                    //turn left
                    if (singleUseKey(Keys.Left) || newGamePadState.DPad.Left == ButtonState.Pressed)
                    {
                        if (activeCamera.getStillToTurnCounterClockwise() <= 0)
                        {
                            activeCamera.setStillToTurnCounterClockwise();
                            PlayerDirection tempDir = GameMap.getPlayerDirection();
                            if (tempDir == PlayerDirection.NORTH)
                            {
                                GameMap.setPlayerDirection(PlayerDirection.WEST);
                            }
                            else
                            {
                                tempDir--;
                                GameMap.setPlayerDirection((PlayerDirection)tempDir);
                            }

                            if (drawBox)
                                drawBox = false;
                        }
                    }

                    //turn right
                    if (singleUseKey(Keys.Right) || newGamePadState.DPad.Right == ButtonState.Pressed)
                    {
                        if (activeCamera.getStillToTurnClockwise() <= 0)
                        {
                            activeCamera.setStillToTurnClockwise();
                            PlayerDirection tempDir = GameMap.getPlayerDirection();
                            if (tempDir == PlayerDirection.WEST)
                            {
                                GameMap.setPlayerDirection(PlayerDirection.NORTH);
                            }
                            else
                            {
                                tempDir++;
                                GameMap.setPlayerDirection((PlayerDirection)tempDir);
                            }

                            if (drawBox)
                                drawBox = false;
                        }
                    }

                    //check the next square
                    if (singleUseKey(Keys.Space) || (newGamePadState.Buttons.A == ButtonState.Pressed && singleUseButton()))
                    {
                        //only check for items if you are not moving
                        if (activeCamera.getStillToMove() <= 0)
                        {
                            if (drawBox)
                            {
                                drawBox = false;
                            }
                            else
                            {
                                checkNextGridItem();
                                drawBox = true;
                            }
                        }
                    }
                }

                else
                {
                    //pad
                    if (newGamePadState.IsConnected)
                    {
                        //look horizontally
                        activeCamera.lookHorizontal(newGamePadState.ThumbSticks.Right.X);
                        //look vertically
                        activeCamera.lookVertical(newGamePadState.ThumbSticks.Right.Y);

                        //up
                        if (newGamePadState.DPad.Up == ButtonState.Pressed)
                        {
                            activeCamera.moveUp();
                        }

                        //down
                        if (newGamePadState.DPad.Down == ButtonState.Pressed)
                        {
                            activeCamera.moveDown();
                        }

                        //forward and back
                        activeCamera.moveForward(newGamePadState.ThumbSticks.Left.Y);
                        //strafe left and right
                        activeCamera.strafe(newGamePadState.ThumbSticks.Left.X);

                        //reset the camera
                        if (newGamePadState.Buttons.LeftStick == ButtonState.Pressed)
                        {
                            activeCamera.resetCam();
                        }
                    }

                    //keyboard

                    //up
                    if (newKeyboardState.IsKeyDown(Keys.Z))
                    {
                        activeCamera.moveUp();
                    }

                    //down
                    if (newKeyboardState.IsKeyDown(Keys.X))
                    {
                        activeCamera.moveDown();
                    }

                    //forward
                    if (newKeyboardState.IsKeyDown(Keys.W))
                    {
                        activeCamera.moveForward();
                    }

                    //back
                    if (newKeyboardState.IsKeyDown(Keys.S))
                    {
                        activeCamera.moveBack();
                    }

                    //strafe left 
                    if (newKeyboardState.IsKeyDown(Keys.A))
                    {
                        activeCamera.strafeLeft();
                    }

                    //strafe right
                    if (newKeyboardState.IsKeyDown(Keys.D))
                    {
                        activeCamera.strafeRight();
                    }

                    //look up
                    if (newKeyboardState.IsKeyDown(Keys.Up))
                    {
                        activeCamera.lookUp();
                    }

                    //look down
                    if (newKeyboardState.IsKeyDown(Keys.Down))
                    {
                        activeCamera.lookDown();
                    }

                    //look right
                    if (newKeyboardState.IsKeyDown(Keys.Right))
                    {
                        activeCamera.lookRight();
                    }

                    //look left
                    if (newKeyboardState.IsKeyDown(Keys.Left))
                    {
                        activeCamera.lookLeft();
                    }


                    //reset the camera
                    if (singleUseKey(Keys.R) || newGamePadState.Buttons.LeftStick == ButtonState.Pressed)
                    {
                        activeCamera.resetCam();
                    }
                }
            }
           

            oldKeyBoardState = newKeyboardState;
            oldGamePadState = newGamePadState;
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            //fps = 1 / (float)gameTime.ElapsedGameTime.TotalSeconds;

            //GraphicsDevice.Clear(Color.CornflowerBlue);
            GraphicsDevice.Clear(ClearOptions.Target | ClearOptions.DepthBuffer, Color.Black, 1.0f, 0);

            // TODO: Add your drawing code here
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;

            if (currentState == GameStates.INTRO)
            {
                    spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);
                    spriteBatch.Draw(intro, new Vector2(0, 0), Color.White);

                    spriteBatch.End();
                
            }

            else
            {



                if (imageShader)
                {
                    Texture2D image = renderImageToTexture(renderImage);

                    GraphicsDevice.Clear(Color.Black);

                    

                    spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);


                    //set the current technique
                    postEffect.CurrentTechnique = postEffect.Techniques[postTechniques[currentPostTech]];
                    postEffect.Begin();
                    //loop through each pass in the technique
                    for (int i = 0; i < postEffect.CurrentTechnique.Passes.Count; i++)
                    {
                        postEffect.CurrentTechnique.Passes[i].Begin();
                        //draw the processed render target to the screen
                        spriteBatch.Draw(image, new Rectangle(0, 0, graphics.GraphicsDevice.Viewport.Width, graphics.GraphicsDevice.Viewport.Height), Color.White);

                        /*
                        int posX = 0, posY = 0;
                        for(int x = 0; x < 4; x++)
                        {
                            for (int y = 0; y < 4; y++)
                            {
                                spriteBatch.Draw(image, new Rectangle(posX, posY, 200, 150), Color.White);
                                posY += 150;
                            }
                            posY = 0;
                            posX += 200;
                        }
                         */
                        postEffect.CurrentTechnique.Passes[i].End();
                    }
                    postEffect.End();

                    spriteBatch.End();
                }

                else
                {
                    if (drawWireframe)
                        graphics.GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;
                    else
                        graphics.GraphicsDevice.RenderState.FillMode = FillMode.Solid;

                    if (useCustomShader)
                        drawWithShader();
                    else
                        drawBasic();
                }



                if (drawMap)
                    drawHUDMap();

                if (showDebugText)
                    writeDebugText();

                if (drawBox)
                    drawTextBox();

            }

         


            base.Draw(gameTime);
        
        }

        public Texture2D renderImageToTexture(RenderTarget2D renderImage)
        {
         
            //set the render target to the render image
            GraphicsDevice.SetRenderTarget(0, renderImage);

            //draw the models which are affected by the shader
            GraphicsDevice.Clear(ClearOptions.Target | ClearOptions.DepthBuffer, Color.DodgerBlue, 1.0f, 0);

            if (useCustomShader)
                drawWithShader();
            else
                drawBasic();

            //revert back to the screen
            GraphicsDevice.SetRenderTarget(0, null);

            //return the image
            return renderImage.GetTexture();

        }

        public void drawBasic()
        {
            

            //xna model, scale rotate then translate
            TileType currentTile;
            MapItem currentItem;
            float currentXPosition = 0;
            float currentYPosition = 0;
            float currentZPosition = 0;

            int tempWidth = GameMap.getMaxWidth(), tempHeight = GameMap.getMaxHeight();

            
            for (int x = 0; x < tempWidth; x++)
            {
                for (int y = 0; y < tempHeight; y++)
                {
                    currentItem = (MapItem)GameMap.getItemTile(x, y);

                    switch (currentItem)
                    {
                        case MapItem.STAIRS:
                            //stairs
                            Matrix[] stairsTransforms = new Matrix[stairs.Bones.Count];
                            stairs.CopyAbsoluteBoneTransformsTo(stairsTransforms);
                            foreach (ModelMesh mesh in stairs.Meshes)
                            {
                                
                                foreach (BasicEffect effect in mesh.Effects)
                                {                                   
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;                                    

                                    Matrix worldScale = Matrix.CreateScale(0.1f);
                                    worldScale *= Matrix.CreateScale(1.0f, 1.5f, 1.0f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(180.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-0.5f, 8.5f, 0.0f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * stairsTransforms[mesh.ParentBone.Index] * worldMatrix;

                                    effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    

                                }

                                mesh.Draw();
                            }
                            break;
                    }


                    currentXPosition += 16.0f;


                }

                currentXPosition = 0;
                currentZPosition += 16.0f;


            }

            currentXPosition = 0;
            currentYPosition = 0;
            currentZPosition = 0;


            //char
            if (gamePlayer.getDrawModel())
            {
                Matrix[] charTransforms = new Matrix[character.Bones.Count];
                character.CopyAbsoluteBoneTransformsTo(charTransforms);
                foreach (ModelMesh mesh in character.Meshes)
                {
                    foreach (BasicEffect effect in mesh.Effects)
                    {
                        effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                        effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                        effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                        effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                        effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                        effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                        effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                        effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                        effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                        effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                        effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                        effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;



                        Matrix worldScale = Matrix.CreateScale(3.0f);
                        Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(gamePlayer.getModelYRotation())) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                        worldMatrix = Matrix.CreateTranslation(gamePlayer.getModelPosition().X, gamePlayer.getModelPosition().Y, gamePlayer.getModelPosition().Z);
                        effect.Projection = projectionMatrix;
                        effect.View = activeCamera.getViewMatrix();
                        effect.World = worldScale * rotationMatrix * charTransforms[mesh.ParentBone.Index] * worldMatrix;

                       

                            effect.FogEnabled = foggy.show;
                            effect.FogStart = foggy.near;
                            effect.FogEnd = foggy.far;
                            effect.FogColor = foggy.colour;
                        

                    }

                    mesh.Draw();
                }
            }

                                    

            for (int x = 0; x < tempWidth; x++)
            {
                for (int y = 0; y < tempHeight; y++)
                {
                    


                    // Trace.WriteLine("Current Tile: " + currentTile);
                    currentTile = (TileType)GameMap.getTile(x, y);
                    //currentTile = (TileType)GameMap.getRandomTile(x, y);

                    switch (currentTile)
                    {
                        case TileType.EMPTY:
                            break;
                            
                        case TileType.DEADEND_W:
                            Matrix[] deadEndTransforms = new Matrix[deadEndModel.Bones.Count];
                            deadEndModel.CopyAbsoluteBoneTransformsTo(deadEndTransforms);

                            foreach (ModelMesh mesh in deadEndModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {                                   


                                   
                                    Matrix worldScale = Matrix.CreateScale(0.04f);
                                    worldScale *= Matrix.CreateScale(1.0f, 1.0f, 1.17f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(180.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(0.0f, 30.5f, 0.0f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * deadEndTransforms[mesh.ParentBone.Index] * worldMatrix;

                                    
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;

                                  
                                   
                                }

                                mesh.Draw();




                            }


                            break;
                            
                        case TileType.JUNCTION_N:
                            Matrix[] junctionNTransforms = new Matrix[junctionModel.Bones.Count];
                            junctionModel.CopyAbsoluteBoneTransformsTo(junctionNTransforms);

                            foreach (ModelMesh mesh in junctionModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    Matrix worldScale = Matrix.CreateScale(0.034f);
                                    worldScale *= Matrix.CreateScale(1.35f, 0.92f, 1.45f);
                                    Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(270.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(0.5f, 3.0f, 2.0f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                   // worldMatrix = Matrix.CreateTranslation(0, 0, 0);
                                    
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * junctionNTransforms[mesh.ParentBone.Index] * worldMatrix;


                                    effect.FogEnabled = foggy.show;
                                    effect.FogStart = foggy.near;
                                    effect.FogEnd = foggy.far;
                                    effect.FogColor = foggy.colour;


                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;
                                }
                                mesh.Draw();
                            }
                            break;
        
                        case TileType.DEADEND_N:
                            Matrix[] deadEndNTransforms = new Matrix[deadEndModel.Bones.Count];
                            deadEndModel.CopyAbsoluteBoneTransformsTo(deadEndNTransforms);

                            foreach (ModelMesh mesh in deadEndModel.Meshes)
                            {

                                foreach (BasicEffect effect in mesh.Effects)
                                {

                                    effect.EnableDefaultLighting();
                                    Matrix worldScale = Matrix.CreateScale(0.04f);
                                    worldScale *= Matrix.CreateScale(1.0f, 1.0f, 1.17f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(90.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-0.5f, 30.5f, 0.0f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * deadEndNTransforms[mesh.ParentBone.Index] * worldMatrix;
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);

                                    
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;
                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.DEADEND_S:
                            Matrix[] deadEndSTransforms = new Matrix[deadEndModel.Bones.Count];
                            deadEndModel.CopyAbsoluteBoneTransformsTo(deadEndSTransforms);

                            foreach (ModelMesh mesh in deadEndModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    Matrix worldScale = Matrix.CreateScale(0.04f);
                                    worldScale *= Matrix.CreateScale(1.0f, 1.0f, 1.17f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(270.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-0.5f, 30.5f, 0.0f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * deadEndSTransforms[mesh.ParentBone.Index] * worldMatrix;
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);

                                    
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;
                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.STRAIGHT_H:
                            Matrix[] straightTransforms = new Matrix[straightModel.Bones.Count];
                            straightModel.CopyAbsoluteBoneTransformsTo(straightTransforms);

                            foreach (ModelMesh mesh in straightModel.Meshes)
                            {

                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    //effect.PreferPerPixelLighting = true;
                                    Matrix worldScale = Matrix.CreateScale(0.02f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    effect.World = worldScale * rotationMatrix * straightTransforms[mesh.ParentBone.Index] * worldMatrix;

                                   
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    
                                   
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;





                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.STRAIGHT_V:
                            Matrix[] straightVTransforms = new Matrix[straightModel.Bones.Count];
                            straightModel.CopyAbsoluteBoneTransformsTo(straightVTransforms);

                            foreach (ModelMesh mesh in straightModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    Matrix worldScale = Matrix.CreateScale(0.02f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(90.0f));
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    effect.World = worldScale * rotationMatrix * straightVTransforms[mesh.ParentBone.Index] * worldMatrix;


                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;


                                    
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    
                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.CORNER_WS:
                            Matrix[] cornerTransforms = new Matrix[cornerModel.Bones.Count];
                            cornerModel.CopyAbsoluteBoneTransformsTo(cornerTransforms);

                            foreach (ModelMesh mesh in cornerModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    Matrix worldScale = Matrix.CreateScale(0.034f);
                                    worldScale *= Matrix.CreateScale(1.3f, 1.0f, 1.5f);
                                    Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(180.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-38.0f, 51.0f, 2.5f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * cornerTransforms[mesh.ParentBone.Index] * worldMatrix;

                                    
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;

                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.CORNER_EN:
                            Matrix[] cornerWNTransforms = new Matrix[cornerModel.Bones.Count];
                            cornerModel.CopyAbsoluteBoneTransformsTo(cornerWNTransforms);

                            foreach (ModelMesh mesh in cornerModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    Matrix worldScale = Matrix.CreateScale(0.034f);
                                    worldScale *= Matrix.CreateScale(1.3f, 1.0f, 1.5f);
                                    Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-34.0f, 51.0f, -2.5f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * cornerWNTransforms[mesh.ParentBone.Index] * worldMatrix;
                                    
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;
                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.CORNER_ES:
                            Matrix[] cornerESTransforms = new Matrix[cornerModel.Bones.Count];
                            cornerModel.CopyAbsoluteBoneTransformsTo(cornerESTransforms);

                            foreach (ModelMesh mesh in cornerModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    Matrix worldScale = Matrix.CreateScale(0.034f);
                                    worldScale *= Matrix.CreateScale(1.5f, 1.0f, 1.5f);
                                    Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(270.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-33.0f, 51.0f, 2.5f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * cornerESTransforms[mesh.ParentBone.Index] * worldMatrix;

                                   
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;
                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.CORNER_WN:
                            Matrix[] cornerENTransforms = new Matrix[cornerModel.Bones.Count];
                            cornerModel.CopyAbsoluteBoneTransformsTo(cornerENTransforms);


                            foreach (ModelMesh mesh in cornerModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    Matrix worldScale = Matrix.CreateScale(0.034f);
                                    worldScale *= Matrix.CreateScale(1.5f, 1.0f, 1.5f);
                                    Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(90.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-38.0f, 51.0f, -2.5f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * cornerENTransforms[mesh.ParentBone.Index] * worldMatrix;
                                   
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;
                                }
                                mesh.Draw();
                            }
                            break;


                        case TileType.FOURWAY:
                            Matrix[] fourwayTransforms = new Matrix[fourwayModel.Bones.Count];
                            fourwayModel.CopyAbsoluteBoneTransformsTo(fourwayTransforms);

                            foreach (ModelMesh mesh in fourwayModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    Matrix worldScale = Matrix.CreateScale(0.034f);
                                    worldScale *= Matrix.CreateScale(1.3f, 1.0f, 1.5f);
                                    Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(5.0f, 8.5f, -0.25f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * fourwayTransforms[mesh.ParentBone.Index] * worldMatrix;
                                   
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;

                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.JUNCTION_S:
                            Matrix[] junctionSTransforms = new Matrix[junctionModel.Bones.Count];
                            junctionModel.CopyAbsoluteBoneTransformsTo(junctionSTransforms);

                            foreach (ModelMesh mesh in junctionModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {                                    
                                    Matrix worldScale = Matrix.CreateScale(0.034f);
                                    worldScale *= Matrix.CreateScale(1.3f, 1.0f, 1.5f);
                                    Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(1.0f, 2.75f, -2.5f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * junctionSTransforms[mesh.ParentBone.Index] * worldMatrix;
                                 
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;

                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.JUNCTION_E:
                            Matrix[] junctionETransforms = new Matrix[junctionModel.Bones.Count];
                            junctionModel.CopyAbsoluteBoneTransformsTo(junctionETransforms);

                            foreach (ModelMesh mesh in junctionModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    Matrix worldScale = Matrix.CreateScale(0.034f);
                                    worldScale *= Matrix.CreateScale(1.5f, 1.0f, 1.5f);
                                    Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(90.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-1.25f, 2.75f, -0.0f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * junctionETransforms[mesh.ParentBone.Index] * worldMatrix;
                                   
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                        effect.FogColor = foggy.colour;
                                    

                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;

                                }
                                mesh.Draw();
                            }
                            break;

                        case TileType.JUNCTION_W:
                            Matrix[] junctionWTransforms = new Matrix[junctionModel.Bones.Count];
                            junctionModel.CopyAbsoluteBoneTransformsTo(junctionWTransforms);

                            foreach (ModelMesh mesh in junctionModel.Meshes)
                            {
                                foreach (BasicEffect effect in mesh.Effects)
                                {
                                    effect.EnableDefaultLighting();
                                    effect.DirectionalLight0.Enabled = basicLighting.lite0Enabled;
                                    effect.DirectionalLight0.DiffuseColor = basicLighting.lite0Colour;
                                    effect.DirectionalLight0.Direction = basicLighting.lite0Direction;
                                    effect.DirectionalLight0.SpecularColor = basicLighting.lite0SpecularColour;

                                    effect.DirectionalLight1.Enabled = basicLighting.lite1Enabled;
                                    effect.DirectionalLight1.DiffuseColor = basicLighting.lite1Colour;
                                    effect.DirectionalLight1.Direction = basicLighting.lite1Direction;
                                    effect.DirectionalLight1.SpecularColor = basicLighting.lite1SpecularColour;

                                    effect.DirectionalLight2.Enabled = basicLighting.lite2Enabled;
                                    effect.DirectionalLight2.DiffuseColor = basicLighting.lite2Colour;
                                    effect.DirectionalLight2.Direction = basicLighting.lite2Direction;
                                    effect.DirectionalLight2.SpecularColor = basicLighting.lite2SpecularColour;

                                    effect.Texture = levelTexture;

                                    Matrix worldScale = Matrix.CreateScale(0.034f);
                                    worldScale *= Matrix.CreateScale(1.5f, 1.0f, 1.5f);
                                    Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(270.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(5.0f, 2.75f, -0.0f);
                                    effect.Projection = projectionMatrix;
                                    effect.View = activeCamera.getViewMatrix();
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    //worldMatrix = Matrix.CreateTranslation(0, 0, 0.0f);
                                    worldMatrix *= translateMatrix;
                                    effect.World = worldScale * rotationMatrix * junctionWTransforms[mesh.ParentBone.Index] * worldMatrix;
                                    
                                        effect.FogEnabled = foggy.show;
                                        effect.FogStart = foggy.near;
                                        effect.FogEnd = foggy.far;
                                       
                                        effect.FogColor = foggy.colour;
                                    

                                }
                                mesh.Draw();
                            }
                            break;
                            


                    }
                    currentXPosition += 16.0f;


                }

                currentXPosition = 0;
                currentZPosition += 16.0f;
            }
        }

        public void drawWithShader()
        {
            //xna model, scale rotate then translate
            TileType currentTile;
            MapItem currentItem;

            float currentXPosition = 0;
            float currentYPosition = 0;
            float currentZPosition = 0;

            int tempWidth = GameMap.getMaxWidth(), tempHeight = GameMap.getMaxHeight();

            for (int x = 0; x < tempWidth; x++)
            {
                for (int y = 0; y < tempHeight; y++)
                {
                    currentItem = (MapItem)GameMap.getItemTile(x, y);

                    
                    switch (currentItem)
                    { 
                        case MapItem.STAIRS:
                            //stairs
                            Matrix[] stairsTransforms = new Matrix[stairs.Bones.Count];
                            stairs.CopyAbsoluteBoneTransformsTo(stairsTransforms);
                            foreach (ModelMesh mesh in stairs.Meshes)
                            {
                                foreach (ModelMeshPart part in mesh.MeshParts)
                                {
                                    part.Effect = testEffect;

                                    Matrix worldScale = Matrix.CreateScale(0.1f);
                                    worldScale *= Matrix.CreateScale(1.0f, 1.5f, 1.0f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(180.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-0.5f, 8.5f, 0.0f);
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    worldMatrix *= translateMatrix;
                                    testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * stairsTransforms[mesh.ParentBone.Index] * worldMatrix);
                                    testEffect.Parameters["View"].SetValue(activeCamera.getViewMatrix());
                                    testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                    Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * stairsTransforms[mesh.ParentBone.Index] * worldMatrix));
                                    testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                    testEffect.Parameters["ModelTexture"].SetValue(stairTexture);
                                    testEffect.Parameters["NormalMap"].SetValue(stairTextureNormal);
                                    testEffect.Parameters["Bumpiness"].SetValue(0.05f);

                                }

                                mesh.Draw();
                            }
                        break;
                    }
                  
                              
                    currentXPosition += 16.0f;


                }

                currentXPosition = 0;
                currentZPosition += 16.0f;

            
            }

            currentXPosition = 0;
            currentYPosition = 0;
            currentZPosition = 0;

            
            
            //char
            if (gamePlayer.getDrawModel())
            {
                Matrix[] charTransforms = new Matrix[character.Bones.Count];
                character.CopyAbsoluteBoneTransformsTo(charTransforms);
                foreach (ModelMesh mesh in character.Meshes)
                {
                    foreach (ModelMeshPart part in mesh.MeshParts)
                    {

                        part.Effect = testEffect;

                        Matrix worldScale = Matrix.CreateScale(3.0f);
                        worldScale *= Matrix.CreateScale(1.0f, 1.0f, 1.0f);
                        Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(gamePlayer.getModelYRotation())) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                        worldMatrix = Matrix.CreateTranslation(gamePlayer.getModelPosition().X, gamePlayer.getModelPosition().Y, gamePlayer.getModelPosition().Z);
                        testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * charTransforms[mesh.ParentBone.Index] * worldMatrix);
                        testEffect.Parameters["View"].SetValue(activeCamera.getViewMatrix());
                        testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                        Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * charTransforms[mesh.ParentBone.Index] * worldMatrix));
                        testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                        testEffect.Parameters["ModelTexture"].SetValue(charTexture);
                        testEffect.Parameters["NormalMap"].SetValue(charTextureNormal);
                        testEffect.Parameters["Bumpiness"].SetValue(0.2f);



                    }

                    mesh.Draw();
                }
            }


            for (int x = 0; x < tempWidth; x++)
            {
                for (int y = 0; y < tempHeight; y++)
                {


                    // Trace.WriteLine("Current Tile: " + currentTile);
                    currentTile = (TileType)GameMap.getTile(x, y);

                    switch (currentTile)
                    {
                        case TileType.EMPTY:
                            break;

                        case TileType.DEADEND_W:
                            Matrix[] deadEndTransforms = new Matrix[deadEndModel.Bones.Count];
                            deadEndModel.CopyAbsoluteBoneTransformsTo(deadEndTransforms);

                            foreach (ModelMesh mesh in deadEndModel.Meshes)
                            {

                                foreach (ModelMeshPart part in mesh.MeshParts)
                                {
                                    part.Effect = testEffect;

                                    Matrix worldScale = Matrix.CreateScale(0.04f);
                                    worldScale *= Matrix.CreateScale(1.0f, 1.0f, 1.17f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(180.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(0.0f, 30.5f, 0.0f);
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    worldMatrix *= translateMatrix;
                                    testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * deadEndTransforms[mesh.ParentBone.Index] * worldMatrix);
                                    testEffect.Parameters["View"].SetValue(activeCamera.getViewMatrix());
                                    testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                    Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * deadEndTransforms[mesh.ParentBone.Index] * worldMatrix));
                                    testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                    testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                    testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                                  
                                }

                                mesh.Draw();




                            }


                            break;
                        
                    case TileType.DEADEND_N:
                    Matrix[] deadEndNTransforms = new Matrix[deadEndModel.Bones.Count];
                    deadEndModel.CopyAbsoluteBoneTransformsTo(deadEndNTransforms);

                    foreach (ModelMesh mesh in deadEndModel.Meshes)
                    {       
                    
                        foreach (ModelMeshPart part in mesh.MeshParts)
                        {
                                    part.Effect = testEffect;

                                    Matrix worldScale = Matrix.CreateScale(0.04f);
                                    worldScale *= Matrix.CreateScale(1.0f, 1.0f, 1.17f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(90.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(-0.5f, 30.5f, 0.0f);
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    worldMatrix *= translateMatrix;
                                    testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * deadEndNTransforms[mesh.ParentBone.Index] * worldMatrix);
                                    testEffect.Parameters["View"].SetValue(activeCamera.getViewMatrix());
                                    testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                    Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * deadEndNTransforms[mesh.ParentBone.Index] * worldMatrix));
                                    testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                    testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                    testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                                 
                         }

                        mesh.Draw();
                    }
                    break;
                           
                    case TileType.DEADEND_S:
                    Matrix[] deadEndSTransforms = new Matrix[deadEndModel.Bones.Count];
                    deadEndModel.CopyAbsoluteBoneTransformsTo(deadEndSTransforms);

                    foreach (ModelMesh mesh in deadEndModel.Meshes)
                    {
                        foreach (ModelMeshPart part in mesh.MeshParts)
                        {
                            part.Effect = testEffect;

                            Matrix worldScale = Matrix.CreateScale(0.04f);
                            worldScale *= Matrix.CreateScale(1.0f, 1.0f, 1.17f);
                            Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(270.0f));
                            Matrix translateMatrix = Matrix.CreateTranslation(-0.5f, 30.5f, 0.0f);
                            worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                            worldMatrix *= translateMatrix;
                            testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * deadEndSTransforms[mesh.ParentBone.Index] * worldMatrix);
                            testEffect.Parameters["View"].SetValue(activeCamera.getViewMatrix());
                            testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                            Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * deadEndSTransforms[mesh.ParentBone.Index] * worldMatrix));
                            testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                            testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                            testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                        
                        }

                        mesh.Draw();
                    }
                    break;
                        
                        case TileType.STRAIGHT_H:
                            Matrix[] straightTransforms = new Matrix[straightModel.Bones.Count];
                            straightModel.CopyAbsoluteBoneTransformsTo(straightTransforms);

                            foreach (ModelMesh mesh in straightModel.Meshes)
                            {
                                foreach (ModelMeshPart part in mesh.MeshParts)
                                {
                                    part.Effect = testEffect;
                                    Matrix worldScale = Matrix.CreateScale(0.02f);
                                    Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                    Matrix translateMatrix = Matrix.CreateTranslation(0.0f, 0.0f, 0.0f);
                                    worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                    worldMatrix *= translateMatrix;
                                    testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * straightTransforms[mesh.ParentBone.Index] * worldMatrix);
                                    testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                    testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                    Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * straightTransforms[mesh.ParentBone.Index] * worldMatrix));
                                    testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                    testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                    testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                                   

                                }
                                mesh.Draw();
                            }
                            break;
                        
                    case TileType.STRAIGHT_V:
                        Matrix[] straightVTransforms = new Matrix[straightModel.Bones.Count];
                        straightModel.CopyAbsoluteBoneTransformsTo(straightVTransforms);

                        foreach (ModelMesh mesh in straightModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.02f);
                                Matrix rotationMatrix = Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(90.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(0.0f, 0.0f, 0.0f);
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * straightVTransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * straightVTransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                                

                            }

                            mesh.Draw();
                        }
                        break;
                            
                    case TileType.CORNER_WS:
                        Matrix[] cornerTransforms = new Matrix[cornerModel.Bones.Count];
                        cornerModel.CopyAbsoluteBoneTransformsTo(cornerTransforms);

                        foreach (ModelMesh mesh in cornerModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.034f);
                                worldScale *= Matrix.CreateScale(1.3f, 1.0f, 1.5f);
                                Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(180.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(-38.0f, 51.0f, 2.5f);
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * cornerTransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * cornerTransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                                

                            }

                            mesh.Draw();
                        }
                        break;
                            
                    case TileType.CORNER_EN:
                        Matrix[] cornerWNTransforms = new Matrix[cornerModel.Bones.Count];
                        cornerModel.CopyAbsoluteBoneTransformsTo(cornerWNTransforms);

                        foreach (ModelMesh mesh in cornerModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.034f);
                                worldScale *= Matrix.CreateScale(1.3f, 1.0f, 1.5f);
                                Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(-34.0f, 51.0f, -2.5f);
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * cornerWNTransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * cornerWNTransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                              
                            }

                            
                            mesh.Draw();
                        }
                        break;

                    case TileType.CORNER_ES:
                        Matrix[] cornerESTransforms = new Matrix[cornerModel.Bones.Count];
                        cornerModel.CopyAbsoluteBoneTransformsTo(cornerESTransforms);

                        foreach (ModelMesh mesh in cornerModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {                               
                                    
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.034f);
                                worldScale *= Matrix.CreateScale(1.5f, 1.0f, 1.5f);
                                Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(270.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(-33.0f, 51.0f, 2.5f);
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * cornerESTransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * cornerESTransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                                

                            }

                           
                            mesh.Draw();
                        }
                        break;

                    case TileType.CORNER_WN:
                        Matrix[] cornerENTransforms = new Matrix[cornerModel.Bones.Count];
                        cornerModel.CopyAbsoluteBoneTransformsTo(cornerENTransforms);

                            
                        foreach (ModelMesh mesh in cornerModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.034f);
                                worldScale *= Matrix.CreateScale(1.5f, 1.0f, 1.5f);
                                Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(90.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(-38.0f, 51.0f, -2.5f);
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * cornerENTransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * cornerENTransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                 

                            }

                          
                            mesh.Draw();
                        }
                        break;
                        
                            
                    case TileType.FOURWAY:
                        Matrix[] fourwayTransforms = new Matrix[fourwayModel.Bones.Count];
                        fourwayModel.CopyAbsoluteBoneTransformsTo(fourwayTransforms);

                        foreach (ModelMesh mesh in fourwayModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.034f);
                                worldScale *= Matrix.CreateScale(1.3f, 1.0f, 1.5f);
                                Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(5.0f, 8.5f, -0.25f);
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * fourwayTransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * fourwayTransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                               
                            }
                          
                            mesh.Draw();
                        }
                        break;

                    case TileType.JUNCTION_S:
                        Matrix[] junctionSTransforms = new Matrix[junctionModel.Bones.Count];
                        junctionModel.CopyAbsoluteBoneTransformsTo(junctionSTransforms);

                        foreach (ModelMesh mesh in junctionModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.034f);
                                worldScale *= Matrix.CreateScale(1.3f, 1.0f, 1.5f);
                                Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(1.0f, 2.75f, -2.5f);
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * junctionSTransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * junctionSTransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                               
                            }
                                                       
                            mesh.Draw();
                        }
                        break;


                    case TileType.JUNCTION_N:
                        Matrix[] junctionNTransforms = new Matrix[junctionModel.Bones.Count];
                        junctionModel.CopyAbsoluteBoneTransformsTo(junctionNTransforms);

                        foreach (ModelMesh mesh in junctionModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.034f);
                                worldScale *= Matrix.CreateScale(1.35f, 0.92f, 1.45f);
                                Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(270.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(0.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(0.5f, 3.0f, 2.0f);
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * junctionNTransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * junctionNTransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                                
                               
                            }
                            mesh.Draw();
                        }
                        break;

                    case TileType.JUNCTION_E:
                        Matrix[] junctionETransforms = new Matrix[junctionModel.Bones.Count];
                        junctionModel.CopyAbsoluteBoneTransformsTo(junctionETransforms);

                        foreach (ModelMesh mesh in junctionModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.034f);
                                worldScale *= Matrix.CreateScale(1.5f, 1.0f, 1.5f);
                                Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(90.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(-1.25f, 2.75f, -0.0f);
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * junctionETransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * junctionETransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                                
                            }
                           
                            mesh.Draw();
                        }
                        break;

                    case TileType.JUNCTION_W:
                        Matrix[] junctionWTransforms = new Matrix[junctionModel.Bones.Count];
                        junctionModel.CopyAbsoluteBoneTransformsTo(junctionWTransforms);

                        foreach (ModelMesh mesh in junctionModel.Meshes)
                        {
                            foreach (ModelMeshPart part in mesh.MeshParts)
                            {
                                part.Effect = testEffect;
                                Matrix worldScale = Matrix.CreateScale(0.034f);
                                worldScale *= Matrix.CreateScale(1.5f, 1.0f, 1.5f);
                                Matrix rotationMatrix = Matrix.CreateRotationX(MathHelper.ToRadians(90.0f)) * Matrix.CreateRotationY(MathHelper.ToRadians(0.0f)) * Matrix.CreateRotationZ(MathHelper.ToRadians(270.0f));
                                Matrix translateMatrix = Matrix.CreateTranslation(5.0f, 2.75f, -0.0f);                                
                                worldMatrix = Matrix.CreateTranslation(currentXPosition, currentYPosition, currentZPosition);
                                worldMatrix *= translateMatrix;
                                testEffect.Parameters["World"].SetValue(worldScale * rotationMatrix * junctionWTransforms[mesh.ParentBone.Index] * worldMatrix);
                                testEffect.Parameters["View"].SetValue((activeCamera.getViewMatrix()));
                                testEffect.Parameters["Projection"].SetValue(projectionMatrix);
                                Matrix worldInverseTransposeMatrix = Matrix.Transpose(Matrix.Invert(worldScale * rotationMatrix * junctionWTransforms[mesh.ParentBone.Index] * worldMatrix));
                                testEffect.Parameters["WorldInverseTranspose"].SetValue(worldInverseTransposeMatrix);
                                testEffect.Parameters["ModelTexture"].SetValue(levelTexture);
                                testEffect.Parameters["NormalMap"].SetValue(levelTextureNormal);
                                
                            }

                            
                            mesh.Draw();
                        }
                        break;
                    


                    }
                    currentXPosition += 16.0f;


                }

                currentXPosition = 0;
                currentZPosition += 16.0f;

            }
                             
        }

        public void drawHUDMap()
        {      
            //find out where the player is
            int currentX = GameMap.getPlayerX();
            int currentY = GameMap.getPlayerY();

            //get the texture dimensions
            int xGap = whiteRect.Width; //we are counting on the blue and white textures being the same dimensions
            int yGap = whiteRect.Height;

            //set where to draw the map
            int initX = 200;
            int initY = 50;
            Vector2 spritePos = new Vector2(initX, initY);
            
            //begin
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
            
            //loop through the visited tile array
            for (int x = 0; x < 14; x++)
            {
                for (int y = 0; y < 14; y++)
                {
                    //if we have visisted it, we can draw it
                    if (GameMap.getVisitedTile(x, y))
                    {
                        //if the player is on that square, draw it blue, else, make it white
                        if (x == currentX && y == currentY)
                            spriteBatch.Draw(whiteRect, spritePos, Color.Blue);
                            else
                                spriteBatch.Draw(whiteRect, spritePos, Color.White);
                        
                    }
                    //update the draw position
                    spritePos.Y += yGap;
                }
                //update the draw position
                spritePos.Y = initY;
                spritePos.X += xGap;
            }

            //end
            spriteBatch.End();
                     
        }

        public void drawTextBox()
        {
       
            int border = boxOfText.border;            

            //boxOfText.text = "Testing, testing";
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
                //white texture used,makes no difference since we change it anyway            
                spriteBatch.Draw(textBoxTexture, new Rectangle(boxOfText.posX - border, boxOfText.posY - border, boxOfText.width + (border * 2), boxOfText.height + (border*2)), boxOfText.borderColor);
                spriteBatch.Draw(textBoxTexture, new Rectangle(boxOfText.posX, boxOfText.posY, boxOfText.width, boxOfText.height), boxOfText.colour);
                spriteBatch.DrawString(fontToUse, boxOfText.text, boxOfText.positionOfText, boxOfText.textColor);
            spriteBatch.End();
        }

        public void checkNextGridItem()
        {
            drawBox = true;

            playAudioClip(AudioClips.SELECT);

            switch(GameMap.checkForItem())
            {              

                case MapItem.EMPTY:
                {
                    boxOfText.text = "Nothing found";
                }
                break;

                case MapItem.STAIRS:
                {
                    boxOfText.text = "Congratulations, you've reached the next level";

                    playAudioClip(AudioClips.STAIRS);
                    GameMap.nextMap();
                    gamePlayer.initPlayerModel(activeCamera.getCamPosition(), GameMap.getPlayerDirection(), activeCamera.getRestrictedState());
                    CamGame.initCam(Vector3.Zero, Looking.RIGHT, GameMap.getPlayerDirection(), true);
                    
                    //the game is stalled while the audio clip is player
                    waitTime = 0;
                    waitValue = 2500;
                    
                  
                }
                break;
                    
            }
            
        }

        public void playAudioClip(AudioClips clip)
        {
            if (soundOn)
            {
                audioClipInstance[(int)clip].Play();
            }         
        }

        //plays/resumes a song
        //should be noted that in order to play a new song, the old song must be stopped first
        public void playSong(AudioSongs song)
        {
            //check if sound is enabled
            if (soundOn)
            {
                //if the current song is paused, resume it
                if (MediaPlayer.State == MediaState.Paused)
                    MediaPlayer.Resume();

                else
                {
                    //use the media player to play the song passed as a parameter
                    MediaPlayer.Play(fullSongArray[(int)song]);
                    MediaPlayer.IsRepeating = true;
                }
            
            }
        }

        //stops the song which is currently playing
        public void stopSong()
        {
            //check the song is actually playing before trying to stop it
            if (MediaPlayer.State != MediaState.Stopped)
                MediaPlayer.Stop();
        }

        //pauses the current song
        public void pauseSong()
        {
            //check the song is actually playing before trying to pause it
            if (MediaPlayer.State == MediaState.Playing)
                MediaPlayer.Pause();
        }

        //sets the volume of the song currently playing
        public void setSongVolume(float level)
        {
            //make sure the volume level is within range
            if (level > 1.0f)
                level = 1.0f;
            if (level < 0.0f)
                level = 0.0f;

            //volume is based on decibel
            //0.0 mute - 1.0 full
            MediaPlayer.Volume = level;
        }       


    }
}


