﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

/*
0 - Empty
1 - Straight Horizontal
2 - Corner East South
3 - Corner East North
4 - Corner West South
5 - Corner West North
6 - Dead End North
7 - Dead End East
8 - Dead End South
9 - Dead End West
10- Junction Not North
11- Junction Not East
12- Junction Not South
13- Junction Not West
14- Four Way
15 - Straight Vertical
*/

//14*14 map
/*
09 01 01 01 04 00 00 00 00 00 00 00 00 00 
00 00 00 00 15 00 00 00 00 00 00 00 00 00	
00 00 00 00 15 00 00 00 00 06 00 00 00 00
02 01 01 01 14 01 01 01 01 03 00 00 00 00
15 00 00 00 15 00 00 00 00 00 00 00 00 00
15 00 06 00 15 00 00 00 00 00 00 00 00 00
15 00 15 00 15 00 00 00 00 00 00 00 00 00
15 00 13 01 12 01 01 01 01 01 01 02 00 00
15 00 15 00 00 00 00 00 00 00 00 05 01 04
03 01 12 01 01 01 01 01 01 02 00 00 00 15
00 00 00 00 00 00 00 00 00 15 00 00 00 15
04 01 01 01 01 01 01 01 01 05 00 02 01 11
15 00 00 00 00 00 00 00 00 00 00 15 00 15
03 01 01 01 01 01 01 01 01 01 01 05 00 08
 * */

/* Improvements
 * 
 * Load map using XML
 * or even better, create random maps dynamically
 * */

namespace Coursework
{
    //type of tile on the map
    public enum TileType:int 
    {
        EMPTY = 0,
        STRAIGHT_H,
        CORNER_ES,
        CORNER_EN,
        CORNER_WS,
        CORNER_WN,
        DEADEND_N,
        DEADEND_E,
        DEADEND_S,
        DEADEND_W,
        JUNCTION_N,
        JUNCTION_E,
        JUNCTION_S,
        JUNCTION_W,
        FOURWAY,
        STRAIGHT_V
    };

    //direction the player is facing
    public enum PlayerDirection : int
    {
        NORTH = 0,
        EAST,
        SOUTH,
        WEST
    };

    //items which can be found
    public enum MapItem: int
    {
        EMPTY = 0,
        STAIRS
    }



    class Map
    {
        //max dimensions of the map
        public const int maxWidth = 14;
        public const int maxHeight = 14;

        //levels
        private int[,] map1;
        private int[,] map2;
        private int[,] map3;

        //current level
        private int[,] floorPlan; 
        //items of current level
        private int[,] itemMap;
        //HUD map
        private bool[,] visited; //bool used for smaller memory requirement
        //current direction the player is facing in
        private PlayerDirection currentDirection;
        //players x and y position
        public int currentPosX, currentPosY;

        /// <summary>
        /// Initialize each level, set the current level to the first one
        /// Initializes the items for the level and sets up the players map(HUD)
        /// </summary>
        public void initMap()
        {
            //each element of a level array corresponds to a particular tile type

            //level 1
            map1 = new int[maxWidth,maxHeight] 
            { 
                {09, 01, 01, 01, 04, 00, 00, 00, 00, 00, 00, 00, 00, 00},
                {00, 00, 00, 00, 15, 00, 00, 00, 00, 00, 00, 00, 00, 00},	
                {00, 00, 00, 00, 15, 00, 00, 00, 00, 06, 00, 00, 00, 00},
                {02, 01, 01, 01, 14, 01, 01, 01, 01, 05, 00, 00, 00, 00},
                {15, 00, 00, 00, 15, 00, 00, 00, 00, 00, 00, 00, 00, 00},
                {15, 00, 06, 00, 15, 00, 00, 00, 00, 00, 00, 00, 00, 00},
                {15, 00, 15, 00, 15, 00, 00, 00, 00, 00, 00, 00, 00, 00},
                {15, 00, 13, 01, 12, 01, 01, 01, 01, 01, 01, 04, 00, 00},
                {15, 00, 15, 00, 00, 00, 00, 00, 00, 00, 00, 03, 01, 04},
                {03, 01, 12, 01, 01, 01, 01, 01, 01, 04, 00, 00, 00, 15},
                {00, 00, 00, 00, 00, 00, 00, 00, 00, 15, 00, 00, 00, 15},
                {02, 01, 01, 01, 01, 01, 01, 01, 01, 05, 00, 02, 01, 11},
                {15, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 15, 00, 15},
                {03, 01, 01, 01, 01, 01, 01, 01, 01, 01, 01, 05, 00, 08}
            };

            //level 2
            map2 = new int[maxWidth, maxHeight] 
            { 
                {09, 01, 01, 01, 01, 10, 01, 01, 01, 01, 01, 10, 01, 04},
                {00, 00, 00, 00, 00, 15, 00, 00, 00, 00, 00, 15, 00, 15},	
                {00, 02, 01, 01, 01, 12, 01, 01, 01, 04, 00, 15, 00, 15},
                {00, 15, 00, 00, 00, 00, 00, 00, 00, 15, 00, 15, 00, 15},
                {00, 15, 00, 02, 01, 10, 01, 04, 00, 15, 00, 15, 00, 15},
                {00, 15, 00, 15, 00, 15, 00, 15, 00, 15, 00, 15, 00, 15},
                {00, 15, 00, 15, 00, 08, 00, 13, 01, 11, 00, 03, 01, 11},
                {00, 13, 01, 11, 00, 00, 00, 15, 00, 15, 00, 00, 00, 15},
                {00, 15, 00, 15, 00, 00, 00, 15, 00, 15, 00, 08, 00, 15},
                {00, 15, 00, 03, 01, 01, 01, 05, 00, 15, 00, 15, 00, 15},
                {00, 15, 00, 00, 00, 00, 00, 00, 00, 15, 00, 15, 00, 15},
                {00, 03, 01, 01, 01, 10, 01, 01, 01, 05, 00, 15, 00, 15},
                {00, 00, 00, 00, 00, 15, 00, 00, 00, 00, 00, 15, 00, 15},
                {09, 01, 01, 01, 01, 12, 01, 01, 01, 01, 01, 12, 01, 05}
            };

            //level 3
            map3 = new int[maxWidth, maxHeight] 
            { 
                {02, 01, 01, 01, 10, 01, 01, 01, 01, 01, 01, 01, 01, 04},
                {15, 00, 00, 00, 15, 00, 00, 00, 00, 00, 00, 00, 00, 15},	
                {15, 00, 00, 00, 15, 00, 02, 01, 01, 01, 01, 01, 01, 11},
                {15, 00, 06, 00, 15, 00, 15, 00, 00, 00, 00, 00, 00, 15},
                {15, 00, 15, 00, 15, 00, 15, 00, 00, 00, 06, 00, 00, 15},
                {15, 00, 03, 01, 05, 00, 03, 01, 01, 01, 05, 00, 00, 15},
                {15, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 00, 15},
                {15, 00, 02, 01, 01, 01, 10, 01, 01, 01, 04, 00, 00, 15},
                {15, 00, 15, 00, 00, 00, 15, 00, 00, 00, 15, 00, 00, 15},
                {15, 00, 15, 00, 06, 00, 15, 00, 00, 00, 03, 04, 00, 15},
                {15, 00, 15, 00, 15, 00, 15, 00, 00, 00, 00, 15, 00, 15},
                {15, 00, 15, 00, 15, 00, 15, 00, 02, 01, 01, 05, 00, 15},
                {15, 00, 15, 00, 15, 00, 15, 00, 15, 00, 00, 00, 00, 15},
                {03, 01, 12, 01, 05, 00, 08, 00, 03, 01, 01, 01, 01, 05}
            };

            //set to level 1
            floorPlan = map1;

            //work out the items for the level
            //item map works as a layer over the top of the current level, used to store the items on the map
            itemMap = new int[maxWidth, maxHeight];
            initItemMap();

            //set up the players map
            visited = new bool[maxWidth, maxHeight];
            initVisited();

            //set the players position to 0,0 and face east
            currentPosX = 0;
            currentPosY = 0;
            setVisitedTile(currentPosX, currentPosY);
            currentDirection = PlayerDirection.EAST;

            
            //generateMaze();
        }

        /// <summary>
        /// Cycles through the current level and 
        /// assigns items to each tile 
        /// 
        /// only the exit has been included so far but this
        /// function can easily be expanded to include any item
        /// </summary>
        public void initItemMap()
        {
            //loop through the level
            for (int x = 0; x < maxHeight; x++)
            {
                for (int y = 0; y < maxHeight; y++)
                {
                    //the exit is always on the south deadend tile type
                    if (floorPlan[x,y] == (int)TileType.DEADEND_S)
                    {
                        itemMap[x, y] = (int)MapItem.STAIRS;
                    }

                    //no item on this tile
                    else
                    {
                        itemMap[x, y] = (int)MapItem.EMPTY;
                    }
                }
            }
        }


        /// <summary>
        /// Set up the HUD map of the level
        /// </summary>
        public void initVisited()
        {
            for (int x = 0; x < maxHeight; x++)
            {
                for (int y = 0; y < maxHeight; y++)
                {
                    //initializing so all cells are false
                    visited[x, y] = false;
                }
            }
        }

        //takes in an x and y position and returns the corresponding cell value of the current level
        public int getTile(int x, int y)
        {
            return floorPlan[x, y];
        }

        //takes in an x and y position and returns the corresponding cell value from the item layer
        public int getItemTile(int x, int y)
        {
            return itemMap[x, y];
        }

        //takes in an x and y position and checks to see if that cell has been visited
        public bool getVisitedTile(int x, int y)
        {
            return visited[x, y];
        }

        //sets the x and y position passed to visited
        public void setVisitedTile(int x, int y)
        {
            visited[x, y] = true;
        }

        /// <summary>
        /// Uses the players current direction to check if the player 
        /// can move to the next cell
        /// 
        /// </summary>
        /// <returns>returns true if the player can move forward</returns>
        public bool mapCollision()
        {
            //use the direction to check the next position
            switch (currentDirection)
            {
                /* for each case
                 * if the player is at the edge of the map and facing outwards
                 *      return false since the can not move any further
                 * check the cell the player will move into next
                 *      if it is 0, it is empty so return false
                 */                
                case PlayerDirection.NORTH:
                    if (currentPosY == 0)
                        return false;
                    if (floorPlan[currentPosY - 1 , currentPosX] == 0)
                        return false;
                break;

                case PlayerDirection.EAST:
                    if (currentPosX == 13)
                        return false;
                    if (floorPlan[currentPosY, currentPosX + 1] == 0)
                        return false;
                break;

                case PlayerDirection.SOUTH:
                    if (currentPosY == 13)
                        return false;
                    if (floorPlan[currentPosY + 1, currentPosX] == 0)
                        return false;
                break;

                case PlayerDirection.WEST:
                    if (currentPosX == 0)
                        return false;
                    if (floorPlan[currentPosY, currentPosX - 1] == 0)
                        return false;
                break;
            }

            //maps ok so check for any items blocking the way
            return checkItemCollision();
        }

        /// <summary>
        /// Uses the players current direction to check which item is in front on them
        /// The player can not move through items
        /// </summary>
        /// <returns>returns false if an item is blocking the way</returns>
        public bool checkItemCollision()
        {
            switch (currentDirection)
            {
                /* for each case
                 * if the player is at the edge of the map and facing outwards
                 *      return false since there is no item
                 * check the cell the player will move into next
                 *      if it is > 0, then there is an item there so return false
                 */  
                case PlayerDirection.NORTH:
                    if (currentPosY == 0)
                        return true;
                    if (itemMap[currentPosY - 1, currentPosX] > 0)
                        return false;
                    break;

                case PlayerDirection.EAST:
                    if (currentPosX == 13)
                        return true;
                    if (itemMap[currentPosY, currentPosX + 1] > 0)
                        return false;
                    break;

                case PlayerDirection.SOUTH:
                    if (currentPosY == 13)
                        return true;
                    if (itemMap[currentPosY + 1, currentPosX] > 0)
                        return false;
                    break;

                case PlayerDirection.WEST:
                    if (currentPosX == 0)
                        return true;
                    if (itemMap[currentPosY, currentPosX - 1] > 0)
                        return false;
                    break;
            }

            //if the code reaches here then no items have been detected, so return true
            return true;
        }

        //this function is the same as checkItemCollision however it returns the value of the item instead of a bool
        //the previous function is used for general collision detection were as this one
        //is used when the user wants to interact with the items on the map
        public MapItem checkForItem()
        {
            //return the item value if there is an item
            switch (currentDirection)
            {
                case PlayerDirection.NORTH:
                    if (currentPosY == 0)
                        return MapItem.EMPTY;
                    if (itemMap[currentPosY - 1, currentPosX] > 0)
                        return (MapItem)itemMap[currentPosY - 1, currentPosX];
                    break;

                case PlayerDirection.EAST:
                    if (currentPosX == 13)
                        return MapItem.EMPTY;
                    if (itemMap[currentPosY, currentPosX + 1] > 0)
                        return (MapItem)itemMap[currentPosY, currentPosX + 1];
                    break;

                case PlayerDirection.SOUTH:
                    if (currentPosY == 13)
                        return MapItem.EMPTY;
                    if (itemMap[currentPosY + 1, currentPosX] > 0)
                        return (MapItem)itemMap[currentPosY + 1, currentPosX];
                    break;

                case PlayerDirection.WEST:
                    if (currentPosX == 0)
                        return MapItem.EMPTY;
                    if (itemMap[currentPosY, currentPosX - 1] > 0)
                        return (MapItem)itemMap[currentPosY, currentPosX - 1];
                    break;
            }

            //no item found, return empty
            return MapItem.EMPTY;
        }
        
        //get/set the players direction
        public void setPlayerDirection(PlayerDirection dir) {currentDirection = (PlayerDirection)dir; }
        public PlayerDirection getPlayerDirection() { return currentDirection; }
        //get the players x and y position in the level 
        public int getPlayerX() { return currentPosX; }
        public int getPlayerY() { return currentPosY; }
        //return the dimensions of the level (size of the array)
        public int getMaxWidth() { return maxWidth; }
        public int getMaxHeight() { return maxHeight; }

        /// <summary>
        /// When the player moves forward their position in th level changes
        /// This function takes the players current direction and uses it
        /// to update the players position
        /// </summary>
        public void updatePlayerPosition()
        {
            //check the players direction
            switch (currentDirection)
            {
                //update accordingly 
                case PlayerDirection.NORTH:
                    currentPosY--;
                break;
                case PlayerDirection.EAST:
                    currentPosX++;
                break;
                case PlayerDirection.SOUTH:
                    currentPosY++;
                break;
                case PlayerDirection.WEST:
                    currentPosX--;
                break;
            }

            //update the players map so they know where they have been
            setVisitedTile(currentPosX, currentPosY);
        }

        /// <summary>
        /// When the player reaches the exit, a new map is set to the current map
        /// and the player is reset back to the start of the map
        /// 
        /// The game loops the same three levels
        /// 
        /// The function is quite basic and at a later date it would be good to 
        /// allow a custom start position/direction as well as using random maps
        /// instead of repeate ones. Some form of proper end game condition would be good as well
        /// </summary>
        public void nextMap()
        {  
            //check the current level and update to the next map
            if (floorPlan == map3)
                floorPlan = map1;
            else if (floorPlan == map2)
                floorPlan = map3;
            else if (floorPlan == map1)
                floorPlan = map2;

            //reset the items and map
            initItemMap();
            initVisited();

            //set the players start location
            currentPosX = 0;
            currentPosY = 0;
            currentDirection = PlayerDirection.EAST;

            //in a cell so update the map
            setVisitedTile(currentPosX, currentPosY);
        }           
    }
}
