﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Coursework
{
    /// <summary>
    /// This class stores the properties related to the players model
    /// Used to determine the models position, rotation and the direction it is facing
    /// 
    /// The players model should only be drawn if the player is in free camera mode
    /// </summary>
    class PlayerModel
    {
        bool drawModel;
        Vector3 modelPosition;
        float modelRotationY;
        PlayerDirection modelDirection;

        /// <summary>
        /// Initialize the properties of the players model
        /// </summary>
        /// <param name="pos">models x,y,z position</param>
        /// <param name="dir">models current direction</param>
        /// <param name="draw">display the model</param>
        public void initPlayerModel(Vector3 pos, PlayerDirection dir, bool draw)
        {
            modelPosition = pos;
            //when initializing the model, this function is passed the state of the camera
            //so if camera restricted = true, then draw model = false
            //hence the reversal of logic
            if(draw)
                drawModel = false;
            else
                drawModel = true;
            modelDirection = dir;

            setRotation();
        }

        /// <summary>
        /// sets the rotation of the model dependant of the players direction
        /// 
        /// the model can only face one of four direction
        /// </summary>
        public void setRotation()
        {
            //check the players direction
            switch(modelDirection)
            {
                //set the models rotation accordingly 
                case PlayerDirection.NORTH:
                    modelRotationY = 270;
                break;
                case PlayerDirection.EAST:
                    modelRotationY = 180;
                break;
                case PlayerDirection.SOUTH:
                    modelRotationY = 90;
                break;
                case PlayerDirection.WEST:
                    modelRotationY = 0;
                break;
            }
        }

        /// <summary>
        /// updates the position and direction of the model
        /// </summary>
        /// <param name="pos">x,y,z position of the model</param>
        /// <param name="dir">direction of the model</param>
        public void updateModel(Vector3 pos, PlayerDirection dir)
        {
            modelPosition = pos;
            modelDirection = dir;
            setRotation();
        }

        //returns the models xyz position
        public Vector3 getModelPosition() { return modelPosition; }
        //used to decide if the model should be drawn or not
        public bool getDrawModel() { return drawModel; }
        //returns the number of degrees the model should be rotated by
        public float getModelYRotation() { return modelRotationY; }
        //sets the model to draw or not
        public void setDrawModel(bool draw)
        {
            //again, dependant on the camera type
            if (draw)
                drawModel = false;
            else
                drawModel = true;
        }
    }
}
