﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Coursework
{
    /// <summary>
    /// Creates a box of a specified colour with a border around it
    /// Can be any size and be placed any where on the screen
    /// Text can be written to the box area
    /// 
    /// The box can also be animated by changing the various properties
    /// </summary>
    class TextBox
    {
        //position
        private int initPosX = 0, initPosY = 0;
        private int currentPosX = 0, currentPosY = 0;
        //dimensions
        private int initWidth = 0, initHeight = 0;
        private int currentWidth = 0, currentHeight = 0;
        //main colour
        private Color initColour = Color.White, currentColour = Color.White;

        //border width and colour
        private int borderValue = 0;
        private Color borderColour = Color.White;

        //text colour and position
        private Color textColour = Color.White;
        private Vector2 textPos = new Vector2(0,0);
        //text to display        
        String textOut = "";

        /// <summary>
        /// Sets the initial values for the main part of the textbox
        /// </summary>
        /// <param name="xPos">starting x position of the box</param>
        /// <param name="yPos">starting y position of the box</param>
        /// <param name="width">starting width of the box</param>
        /// <param name="height">starting height of the box</param>
        /// <param name="colour">starting colour of the box</param>
        public void initTextBox(int xPos, int yPos, int width, int height, Color colour)
        {
            //set initial values
            initPosX = xPos;
            initPosY = yPos;
            initWidth = width;
            initHeight = height;
            initColour = colour;
            //set the current values to the inital values
            currentPosX = initPosX;
            currentPosY = initPosY;
            currentWidth = initWidth;
            currentHeight = initHeight;
            currentColour = initColour;            
        }

        /// <summary>
        /// Creates a border around the main box
        /// </summary>
        /// <param name="borderGap">size of the border</param>
        /// <param name="colour">colour of the border</param>
        public void initBorder(int borderGap, Color colour)
        {
            //set the values
            borderValue = borderGap;
            borderColour = colour;
        }

        /// <summary>
        /// Sets the colour of the text
        /// Sets the position of the text
        /// </summary>
        /// <param name="colour">colour of the text</param>
        /// <param name="gapX">distance from top of the main box</param>
        /// <param name="gapY">distance from left of the main box</param>
        public void initTextProperties(Color colour, int gapX, int gapY)
        {
            //set the colour
            textColour = colour;
            //use the current position of the main box to calculate the text position
            textPos.X = currentPosX + gapX;
            textPos.Y = currentPosY + gapY;
        }

        //getter/setter methods for position, dimension and colour of the main box
        public int posX { get { return currentPosX; } set { currentPosX = value; } }
        public int posY { get { return currentPosY; } set { currentPosY = value; } }
        public int width { get { return currentWidth; } set { currentWidth  = value; } }
        public int height { get { return currentHeight; } set { currentHeight = value; } }
        public Color colour { get { return currentColour; } set { currentColour = value; } }
        //getter/setter methods for the border properties
        public int border { get { return borderValue; } set { borderValue = value; } }
        public Color borderColor { get { return borderColour; } set { borderColour = value; } }
        //getter/setter methods for the text properties
        public Color textColor { get { return textColour; } set { textColour = value; } }
        public Vector2 positionOfText { get { return textPos; } set { textPos = value; } }
        public String text { get { return textOut; } set { textOut = value; } }
        
        /// <summary>
        /// resets the position of the box back to its starting position
        /// </summary>
        public void resetBox()
        {
            currentPosX = initPosX;
            currentPosY = initPosY;
            currentWidth = initWidth;
            currentHeight = initHeight;
            currentColour = initColour;
            textOut = "";
        }
    }
}
